@@ -37,6 +37,7 @@
 #include "gdm-greeter-client.h"
 #include "gdm-greeter-panel.h"
 #include "gdm-greeter-login-window.h"
+#include "gdm-user-chooser-widget.h"
 
 #include "gdm-profile.h"
 
@@ -101,12 +102,24 @@ on_reset (GdmGreeterClient  *client,
         gdm_greeter_login_window_reset (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window));
 }
 
+static void
+show_or_hide_user_options (GdmGreeterSession *session,
+                           const char        *username)
+{
+    if (username != NULL && strcmp (username, GDM_USER_CHOOSER_USER_OTHER) != 0) {
+            gdm_greeter_panel_show_user_options (GDM_GREETER_PANEL (session->priv->panel));
+    } else {
+            gdm_greeter_panel_hide_user_options (GDM_GREETER_PANEL (session->priv->panel));
+    }
+}
+
 static void
 on_selected_user_changed (GdmGreeterClient  *client,
                           const char        *text,
                           GdmGreeterSession *session)
 {
         g_debug ("GdmGreeterSession: selected user changed: %s", text);
+        show_or_hide_user_options (session, text);
 }
 
 static void
@@ -240,7 +253,7 @@ on_select_user (GdmGreeterLoginWindow *login_window,
                 const char            *text,
                 GdmGreeterSession     *session)
 {
-        gdm_greeter_panel_show_user_options (GDM_GREETER_PANEL (session->priv->panel));
+        show_or_hide_user_options (session, text);
         gdm_greeter_client_call_select_user (session->priv->client,
                                              text);
 }
@@ -267,6 +280,41 @@ on_start_session (GdmGreeterLoginWindow *login_window,
         gdm_greeter_client_call_start_session_when_ready (session->priv->client, TRUE);
 }
 
+static int
+get_tallest_monitor_at_point (GdkScreen *screen,
+                              int        x,
+                              int        y)
+{
+        GdkRectangle area;
+        GdkRegion *region;
+        int i;
+        int monitor;
+        int n_monitors;
+        int tallest_height;
+
+        tallest_height = 0;
+        n_monitors = gdk_screen_get_n_monitors (screen);
+        monitor = -1;
+        for (i = 0; i < n_monitors; i++) {
+                gdk_screen_get_monitor_geometry (screen, i, &area);
+                region = gdk_region_rectangle (&area);
+
+                if (gdk_region_point_in (region, x, y)) {
+                        if (area.height > tallest_height) {
+                                monitor = i;
+                                tallest_height = area.height;
+                        }
+                }
+                gdk_region_destroy (region);
+        }
+
+        if (monitor == -1) {
+                monitor = gdk_screen_get_monitor_at_point (screen, x, y);
+        }
+
+        return monitor;
+}
+
 static void
 toggle_panel (GdmGreeterSession *session,
               gboolean           enabled)
@@ -281,7 +329,8 @@ toggle_panel (GdmGreeterSession *session,
 
                 display = gdk_display_get_default ();
                 gdk_display_get_pointer (display, &screen, &x, &y, NULL);
-                monitor = gdk_screen_get_monitor_at_point (screen, x, y);
+
+                monitor = get_tallest_monitor_at_point (screen, x, y);
 
                 session->priv->panel = gdm_greeter_panel_new (screen, monitor);
 
