@@ -123,7 +123,7 @@ gdm_parse_language_name (const char *name,
         error = NULL;
         re = g_regex_new ("^(?P<language>[^_.@[:space:]]+)"
                           "(_(?P<territory>[[:upper:]]+))?"
-                          "(.(?P<codeset>[-_0-9a-zA-Z]+))?"
+                          "(\\.(?P<codeset>[-_0-9a-zA-Z]+))?"
                           "(@(?P<modifier>[[:ascii:]]+))?$",
                           0, 0, &error);
         if (re == NULL) {
@@ -481,7 +481,7 @@ select_dirs (const struct dirent *dirent)
                                 struct stat st;
                                 char       *path;
 
-                                path = g_build_filename (GNOMELOCALEDIR, dirent->d_name, NULL);
+                                path = g_build_filename (LIBLOCALEDIR, dirent->d_name, NULL);
                                 if (g_stat (path, &st) == 0) {
                                         mode = st.st_mode;
                                 }
@@ -501,7 +501,7 @@ collect_locales_from_directory (void)
         int             ndirents;
         int             cnt;
 
-        ndirents = scandir (GNOMELOCALEDIR, &dirents, select_dirs, alphasort);
+        ndirents = scandir (LIBLOCALEDIR, &dirents, select_dirs, alphasort);
 
         for (cnt = 0; cnt < ndirents; ++cnt) {
                 add_locale (dirents[cnt]->d_name);
